﻿//-----------------------------------------------------------------------
// <copyright file="DataVisualizationGuiModule.cs" company="dataPartner">
//     Copyright (c) dataPartner s. r. o.  All rights reserved.
// </copyright>
// <author>
//     Vojtech Osvald
// </author>
//-----------------------------------------------------------------------

namespace BioWes.DataVisualization
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics.Contracts;
    using System.Windows.Forms;
    using BioWes.ProjectManager;
    using BioWes.ProjectManager.FileManagement;
    using BioWes.DataVisualization.Commands;
    using dataPartner.ApplicationFramework4;
    using dataPartner.ApplicationFramework4.Commands;
    using dataPartner.ApplicationFramework4.Face;
    using dataPartner.ComponentModel;
    using DevExpress.XtraCharts.Wizard;
    using System.Globalization;

    public class DataVisualizationGuiModule : BaseFaceModule
    {
        #region Property

        private RibbonTabCategory _dataVisualizationTabCategory;
        private Ribbon _moduleRibbon;

        private UserControls.DataChartForm _dataChartForm;

        #endregion

        #region Konstruktor

        public DataVisualizationGuiModule() : base()
        {

        }

        #endregion

        #region Metody

        private void CreateRibbon()
        {
            Contract.Assert(_moduleRibbon == null);

            _moduleRibbon = AppFramework.Face.Ribbon.CreateRibbon();

            // register ShowDataCommand to ribbon of ProtocolManager
            var protocolsTab = _moduleRibbon.Tabs.CreateTab(ProjectManager.Commands.ProjectManagerUINames.ProtocolsTab);
            _dataVisualizationTabCategory = _moduleRibbon.TabCategories.CreateTabCategory(ProjectManager.Commands.ProjectManagerUINames.ProtocolsTabCategory);
            var visualizationGroup = protocolsTab.Groups.CreateGroup(UINames.VisualizationRibbonGroup, 10); // 10 is order of group in another ribbon

            ICommand cmd;
            cmd = new ShowDataCommand();
            cmd.Executed += ShowDataCommand_Executed;
            AppFramework.CommandManager.Commands.Add(cmd);
            visualizationGroup.Commands.Add(cmd);
        }

        private void ShowDataChartForm(IEnumerable<Tuple<DateTime, double>> data)
        {
            if (_dataChartForm == null)
            {
                _dataChartForm = new UserControls.DataChartForm(AppFramework);
                _dataChartForm.Activated += DemoModuleForm_Activated;
                _dataChartForm.Deactivate += DemoModuleForm_Deactivate;
                _dataChartForm.FormClosed += DemoModuleForm_FormClosed;

                ((IServiceRegistry)AppFramework).RegisterService(typeof(UserControls.IDataChartFormProvider), _dataChartForm);
            }

            if (AppFramework.Face.Windows.ContainsKey(UserControls.DataChartForm.FormName))
            {
                IWindow window = AppFramework.Face.Windows[UserControls.DataChartForm.FormName];
                if (window != _dataChartForm)
                {
                    // nemelo by nastat
                    AppFramework.Face.Windows.Remove(UserControls.DataChartForm.FormName);
                    AppFramework.Face.Windows[UserControls.DataChartForm.FormName] = _dataChartForm;
                }
            }
            else
            {
                AppFramework.Face.Windows[UserControls.DataChartForm.FormName] = _dataChartForm;
            }

            _dataChartForm.ReloadData(data);
        }

        private void DemoModuleForm_Activated(object sender, EventArgs e)
        {
            _dataVisualizationTabCategory.Show(RibbonTabCategoryShowOption.ActivatePreviouslySelectedTabOrFirst); // activate DemoModule ribbon category
        }

        private void DemoModuleForm_Deactivate(object sender, EventArgs e)
        {
        }

        private void DemoModuleForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            ((IServiceRegistry)AppFramework).UnregisterService(typeof(UserControls.IDataChartFormProvider));

            if (AppFramework.Face.Windows.ContainsKey(UserControls.DataChartForm.FormName))
            {
                AppFramework.Face.Windows.Remove(UserControls.DataChartForm.FormName);
            }

            _dataChartForm = null;
        }

        #endregion

        #region Events

        private void ShowDataCommand_Executed(object sender, System.ComponentModel.HandledEventArgs e)
        {
            var protocolManager = AppFramework.GetService<IProtocolManagerService>();
            IFileService iFileService = AppFramework.GetService<IFileService>();
            IFileChooser fileChooser = iFileService.CreateFileChooser();
            if (protocolManager != null && protocolManager.SelectedProtocol != null && iFileService != null && fileChooser != null)
            {
                if (protocolManager.SelectedProtocol.UserRights.CanReadFiles)
                {
                    try
                    {
                        fileChooser.Mode = FileSelectorMode.Single;
                        fileChooser.AssignFiles(protocolManager.SelectedProtocol.Files);
                        if (fileChooser.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                        {
                            if (fileChooser.SelectedFile != null && fileChooser.SelectedFile.IsDataFile)
                            {
                                var data = new List<Tuple<DateTime, double>>();

                                try
                                {
                                    using (var dataStream = new System.IO.MemoryStream(fileChooser.SelectedFile.Data))
                                    using (var reader = new System.IO.StreamReader(dataStream))
                                    {
                                        var csvData = reader.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
                                        foreach (var line in csvData)
                                        {
                                            var pair = line.Split(new string[] { "\t" }, StringSplitOptions.RemoveEmptyEntries);
                                            data.Add(new Tuple<DateTime, double>(DateTime.Parse(pair[0], CultureInfo.GetCultureInfo("cs-CZ")), Double.Parse(pair[1])));
                                        }
                                    }

                                }
                                catch
                                {
                                    // v pripade ze neprosla cestina, zkusime datumy rozparsovat pomoci invariant culture
                                    try
                                    {
                                        using (var dataStream = new System.IO.MemoryStream(fileChooser.SelectedFile.Data))
                                        using (var reader = new System.IO.StreamReader(dataStream))
                                        {
                                            var csvData = reader.ReadToEnd().Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);
                                            foreach (var line in csvData)
                                            {
                                                var pair = line.Split(new string[] { "\t" }, StringSplitOptions.RemoveEmptyEntries);
                                                data.Add(new Tuple<DateTime, double>(DateTime.Parse(pair[0], CultureInfo.InvariantCulture), Double.Parse(pair[1])));
                                            }
                                        }

                                    }
                                    catch
                                    {
                                        AppFramework.Face.ShowMessageBox(Resources.Messages.ShowDataCommand_BadFormatDataFile, null, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                                        return;
                                    }
                                }

                                ShowDataChartForm(data);

                                ChartWizard chartWizard = new ChartWizard(((UserControls.IDataChartFormProvider)_dataChartForm).GetChartControl());
                                chartWizard.ShowDialog();
                            }
                            else
                            {
                                AppFramework.Face.ShowMessageBox(Resources.Messages.ShowDataCommand_NotaDataFile, null, MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        AppFramework.Face.ShowException(Resources.Messages.ShowDataCommand_UnhandledException, ex, true, System.Windows.Forms.MessageBoxButtons.OK);
                    }
                }
                else
                {
                    AppFramework.Face.ShowMessageBox(Resources.Messages.ShowDataCommand_NotSufficientUserRights, Resources.Messages.MessageBoxHeader_Error, System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Error);
                }
            }
        }

        #endregion

        #region Overrides

        public override void Configure()
        {
            base.Configure();
        }

        public override void Run()
        {
            base.Run();

            CreateRibbon();
            AppFramework.Face.Ribbon.Merge(_moduleRibbon);
        }

        public override void Stop()
        {
            AppFramework.Face.Ribbon.Unmerge(_moduleRibbon);

            base.Stop();
        }

        protected override void OnActivated()
        {
            base.OnActivated();
        }

        protected override void OnDeactivated()
        {
            base.OnDeactivated();
        }

        #endregion
    }
}
